c----------------------------------------------------------------------
      program AER2CAL
c----------------------------------------------------------------------
c
c --- AER2CAL   Version: 1.21            Level: 100301            MAIN
c
c ---           D. Strimaitis
c
c     Copyright (c) 2006-2010 by Exponent, Inc.         
c
c
c  PURPOSE:     Main program for converting concentration data produced
c               by AERMOD (POSTFILE) to CALPUFF CONC.DAT.
c
c  UPDATES:
c    v1.2 Level 091104  -->  v1.21 Level 100301  (DGS)
c         (1) Generalize AERMOD reads so that AER2CAL works for both
c             real*4 and real*8 versions of new and old AERMOD files.
c         Modified: COMP
c
c    v1.1 Level 090122  -->  v1.2 Level 091104  (DGS)
c         (1) NEW AERMOD(09292) uses real*8 variables, so change read
c             and then use real*4 part.  This needs to be generalized
c             to be able to process both new and old AERMOD files.
c         Modified: COMP
c
c    v1.04 Level 080926  -->  v1.1 Level 090122  (DGS)
c         (1) Get species name from 'CO POLLUTID' record
c         (2) Allow for '  ' as pathway marker
c         (3) Interpret contents on POSTFILE record(s) and process
c             only files with correct attributes (1-hour CONC)
c         (4) Expand date-time structures that are recognized
c                STARTEND YYYY MM DD HH YYYY MM DD HH
c                STARTEND YYYY MM DD YYYY MM DD
c             and STOP processing if any DAYRANGE records are used.
c             DAYRANGE instructions can introduce gaps in the output
c             which is not supported in CALPOST.
c         Modified: CTRL.A2C
c                   SETUP
c
c    v1.03 Level 070925  -->  v1.04 Level 080926  (DGS)
c         (1) Change default DATUM from NWS-84 to WGS-84
c
c    v1.02 Level 070131  -->  v1.03 Level 070925  (DGS)
c         (1) Restructure how QA problems are communicated to make it
c             easier to port to a DLL for calls from a GUI
c         (2) Add banner to the list file
c         (3) Drop unused subroutines
c         Modified: COMP, SETUP
c         Replaced: CASE320, INCR, JULDAY
c         Removed:  GRDAY
c
c    v1.01 Level 061124  -->  v1.02 Level 070131  (DGS)
c         (1) Fix bug in COMP that allowed an extra 1-hour shift back
c             for all hours except the last hour of each day.  This bug
c             results in CALPOST output labels being off by 1 hour most
c             of the time.
c         (2) Add COMLINE to provide AERMOD control file name on command
c             line, and remove the interactive input section.
c         New:      COMLINE (from CALUTILS)
c         Modified: COMP, SETUP
c
c    v1.0  Level 061122  -->  v1.01 Level 061124  (DGS)
c         (1) Convert receptor coordinates from M to KM
c         (2) Set AERMOD ver,level
c         (3) Restructure comment records
c         (4) Remove version,level from params file (no checks)
c         Modified: PARAMS.A2C
c                   SETUP, WROUT1
c
c  ARGUMENTS:   none
c
c  CALLING ROUTINES:    none
c
c  EXTERNAL ROUTINES:   SETUP, COMP
c-----------------------------------------------------------------------
c --- Run Notes ---
c
c --- Invoke using command line: aer2cal fname
c
c --- where fname is the AERMOD control file used to create the file(s)
c --- being processed.
c ---------------------------------------------------------------------
c --- Include parameters
      include 'params.a2c'

c --- Include common blocks
      include 'qa.a2c'

c --- String for reporting QA issues
      character*132 halt132

c --- set version and level number of program
      ver='1.21'
      level='100301'

c --- Initialize problem string
      halt132(1:18)='NORMAL TERMINATION'
      do i=19,132
         halt132(i:i)=' '
      enddo

c --- SETUP PHASE -- initialization & program setup operations
      write(*,*)
      write(*,*)'SETUP PHASE'
      write(*,*)
      call SETUP(halt132)
      if(halt132(1:6) .NE. 'NORMAL') then
         write(*,*) halt132
         goto 999
      endif

c --- COMPUTATIONAL PHASE -- basic time loop with scientific modules
      write(*,*)
      write(*,*)'COMPUTATION PHASE'
      write(*,*)
      call COMP(halt132)
      if(halt132(1:6) .NE. 'NORMAL') then
         write(*,*) halt132
         goto 999
      endif

c --- TERMINATION PHASE -- program termination functions
      write(*,*)
      write(*,*)'NORMAL TERMINATION'
      write(*,*)
      write(ilst,*)
      write(ilst,*)'NORMAL TERMINATION'
      write(ilst,*)

c --- Close any open files
999   close(ilst)

      stop
      end

c----------------------------------------------------------------------
      subroutine setup(halt132)
c----------------------------------------------------------------------
c
c --- AER2CAL    Version: 1.21     Level: 090122                 SETUP
c                D. Strimaitis
c
c --- PURPOSE:  Prepare information for run
c
c  UPDATES:
c    v1.03 Level 070925  -->  v1.1 Level 090122  (DGS)
c         - Get species name from 'CO POLLUTID' record
c         - Allow for '  ' as pathway marker
c         - Interpret contents on POSTFILE record(s) and process
c           only files with correct attributes (1-hour CONC)
c         - Expand date-time structures that are recognized
c                STARTEND YYYY MM DD HH YYYY MM DD HH
c                STARTEND YYYY MM DD YYYY MM DD
c           and STOP processing if any DAYRANGE records are used.
c           DAYRANGE instructions can introduce gaps in the output
c           which is not supported in CALPOST.
c
c    v1.02 Level 070131  -->  v1.03 Level 070925  (DGS)
c         - Add argument variable HALT132 to control early termination
c         - Derive listfile name from control filename
c         - Add banner to list file
c
c    v1.01 Level 061124  -->  v1.02 Level 070131  (DGS)
c         - Add COMLINE to provide AERMOD control file name on command
c           line, and remove the interactive input section.
c
c    v1.0  Level 061122  -->  v1.01 Level 061124  (DGS)
c         - Convert receptor coordinates from M to KM
c
c --- INPUT:  
c          HALT132 - character*132     - Status string
c
c          Common/QA/
c            ver, level
c
c --- OUTPUT:  
c          HALT132 - character*132     - Status string
c
c          Common/CTRL/
c            nrecin,xrecin,yrecin,zrecin,atitle,aernam,nfil,
c            isyr,ismo,isday,isjul,ishr,ieyr,iemo,ieday,iejul,iehr,
c            aerspec
c
c --- SETUP called by:  main
c --- SETUP calls:      CASE320, JULDAY
c----------------------------------------------------------------------

c --- Include parameter statements
      include 'params.a2c'
      include 'ctrl.a2c'
      include 'qa.a2c'

      character*132 cfname, recfil, lstname, halt132
      character*320 char320, blnk320, ctmp320
      character*2 cpathway
      logical lconc,ldayrange

      do i=1,320
         blnk320(i:i)=' '
      enddo

c --- Open a scratch file for x,y,z,h records (without DISCCART tag)
c --- to facilitate reading the data with a free-read
      open(ixx,status='scratch')

c -----------------
c --- Control file
c -----------------
c --- Get control file for target AERMOD application
c --- From command line
      cfname='AERMOD.INP'
      call COMLINE(cfname)
      klast=LEN_TRIM(cfname)

c --- Create list filename from AERMOD control filename by adding
c --- '.a2c' at the end
      lstname=blnk320(1:132)
      lstname=cfname(1:klast)//'.a2c'
      open(ilst,file=lstname,status='unknown')

c --- Write banner to list file
      write(ilst,5) ver,level
5     format(///,26x,'AER2CAL RUN SUMMARY',/,19x,'VERSION:  ',A12,
     1       ' LEVEL:  ',A12///)

c --- Note
      write(*,*)
      write(*,*)'This utility must be run in the location that'
      write(*,*)'contains the AERMOD application files.  If this'
      write(*,*)'is not the case, file-error messages will be seen!'
      write(*,*)
      write(ilst,*)
      write(ilst,*)'This utility must be run in the location that'
      write(ilst,*)'contains the AERMOD application files.  If this'
      write(ilst,*)'is not the case, file-error messages will be seen!'
      write(ilst,*)

c --- Open control file
      write(*,*)
      write(*,*)'Opening file: ',cfname(1:klast)
      write(*,*)
      write(ilst,*)
      write(ilst,*)'Opening file: ',cfname(1:klast)
      write(ilst,*)
      open(ictl,file=cfname,status='old')

c --- Set counters/flags for prospecting the control file
      nfil=0
      nrecin=0
      nrecfil=0
      ldayrange=.FALSE.
      cpathway='  '
      aerspec='            '

c --- Process each control file record
c ------------------------------------
10    char320=blnk320
      read(ictl,'(a320)',end=100) char320

c --- Skip any line that begins with *
      if(char320(1:1).EQ.'*') goto 10

c --- Create upper-case version of line
      klast=LEN_TRIM(char320)
      call CASE320(ilst,'UPPER',char320,klast,halt132)
      if(halt132(1:6) .NE. 'NORMAL') return

c --- Persist current pathway marker
      if(char320(1:2).NE.'  ') then
         cpathway=char320(1:2)
      else
         char320(1:2)=cpathway
      endif

c --- Transfer title lines to /head/ variable
      if(INDEX(char320,'TITLEONE').NE.0) then
         i1=INDEX(char320,'TITLEONE')+8
         i2=klast
         read(char320(i1:i2),'(a)') atitle(1)
         atitle(1)=ADJUSTL(atitle(1))
      endif
      if(INDEX(char320,'TITLETWO').NE.0) then
         i1=INDEX(char320,'TITLETWO')+8
         i2=klast
         read(char320(i1:i2),'(a)') atitle(2)
         atitle(2)=ADJUSTL(atitle(2))
      endif

c --- Read the Pollutant ID (species)
      if(INDEX(char320,'CO POLLUTID').NE.0) then
         i1=INDEX(char320,'POLLUTID')+8
         i2=klast
         ctmp320=blnk320
         ctmp320(i1:i2)=char320(i1:i2)
         ctmp320=ADJUSTL(ctmp320)
         aerspec=ctmp320(1:12)
      endif

c --- Check model options for just concentrations
      if(INDEX(char320,'CO MODELOPT').NE.0) then
         lconc=.FALSE.
         if(INDEX(char320,'CONC').NE.0) lconc=.TRUE.
         if(INDEX(char320,'DEPOS').NE.0) lconc=.FALSE.
         if(INDEX(char320,'DDEP').NE.0) lconc=.FALSE.
         if(INDEX(char320,'WDEP').NE.0) lconc=.FALSE.
      endif

c --- Check for presence of DAYRANGE (not supported)
      if(INDEX(char320,'ME DAYRANGE').NE.0) ldayrange=.TRUE.

c --- Read the date-time range for the data
      if(INDEX(char320,'ME STARTEND').NE.0) then
         i1=INDEX(char320,'STARTEND')+8
         i2=klast
c ---    Try full record with hours
         read(char320(i1:i2),*,end=19) isyr,ismo,isday,ishr,
     &                                 ieyr,iemo,ieday,iehr
         goto 20
c ---    Try optional record without hours
19       read(char320(i1:i2),*) isyr,ismo,isday,
     &                          ieyr,iemo,ieday
         ishr=1
         iehr=24
20       call JULDAY(ilst,isyr,ismo,isday,isjul,halt132)
         if(halt132(1:6) .NE. 'NORMAL') return
         call JULDAY(ilst,ieyr,iemo,ieday,iejul,halt132)
         if(halt132(1:6) .NE. 'NORMAL') return
      endif

c --- Harvest POSTFILE names
      if(INDEX(char320,'OU POSTFILE').NE.0) then
c ---    Must output 1-hour averages
         if(INDEX(char320,' 1 ').NE.0) then
           i1=INDEX(char320,'UNFORM')+6
c ---      Check for a file unit number at end by looking for a blank
c ---      in the string like "name n" through "name nnn"
           itest1=klast-1
           itest2=klast-2
           itest3=klast-3
           if(char320(itest1:itest1).EQ.' ') then
              klast=itest1-1
           elseif(char320(itest2:itest2).EQ.' ') then
              klast=itest2-1
           elseif(char320(itest3:itest3).EQ.' ') then
              klast=itest3-1
           endif
           i2=klast
           nfil=nfil+1
           if(nfil.GT.mxfil) then
              k=47
              k1=k+1
              k2=k+25
              k3=k2+1
              halt132(1:k)=
     &                'Halted in SETUP:  there are too many file names'
              halt132(k1:k2)=
     &             '   --   See the list file'
              do i=k3,132
                 halt132(i:i)=' '
              enddo
              write(ilst,*)halt132(1:k)
              write(ilst,*)'The limit is set by parameter MXFIL = ',
     &                      mxfil
              return
           endif
           read(char320(i1:i2),'(a)') aernam(nfil)
           aernam(nfil)=ADJUSTL(aernam(nfil))
         endif
      endif

c --- Harvest receptor information
c --- Transfer x,y,z,h records (without DISCCART tag) to scratch file
      if(INDEX(char320,'DISCCART').NE.0) then
         nrecin=nrecin+1
         i1=INDEX(char320,'DISCCART')+8
         i2=klast
         write(ixx,'(a)') char320(i1:i2)
      endif

c --- Harvest filename for the receptors
      if(INDEX(char320,'RE INCLUDED').NE.0) then
         i1=INDEX(char320,'RE INCLUDED')+11
         i2=klast
         nrecfil=nrecfil+1
         read(char320(i1:i2),'(a)') recfil
         recfil=ADJUSTL(recfil)
      endif

c --- Look for more control file records
      goto 10

c --- Done with AERMOD control file
100   close(ictl)

c --- Process receptor file?
      if(nrecfil.GT.1) then
         k=43
         k1=k+1
         k2=k+25
         k3=k2+1
         halt132(1:k)=
     &             'Halted in SETUP:  too many receptor files: '
         halt132(k1:k2)=
     &             '   --   See the list file'
         do i=k3,132
            halt132(i:i)=' '
         enddo
         write(ilst,*)halt132(1:k),nrecfil
         write(ilst,*)'Only 1 receptor file is expected'
         return
      elseif(nrecfil.EQ.0) then
         goto 300
      endif

c -----------------
c --- Receptor file
c -----------------
c --- Transfer x,y,z,h records (without DISCCART tag) to scratch file
      open(ictl,file=recfil,status='old')

c --- Process each receptor file record
c -------------------------------------
110   char320=blnk320
      read(ictl,'(a320)',end=200) char320

c --- Skip any line that begins with *
      if(char320(1:1).EQ.'*') goto 110

c --- Create upper-case version of line
      klast=LEN_TRIM(char320)
      call CASE320(ilst,'UPPER',char320,klast,halt132)
      if(halt132(1:6) .NE. 'NORMAL') return

c --- Transfer receptor records
      if(INDEX(char320,'DISCCART').NE.0) then
         nrecin=nrecin+1
         i1=INDEX(char320,'DISCCART')+8
         i2=klast
         write(ixx,'(a)') char320(i1:i2)
      endif

c --- Look for more receptor records
      goto 110

c --- Done with AERMOD receptor file
200   close(ictl)

c --- Process scratch file records
c --------------------------------
300   rewind(ixx)
      do k=1,nrecin
         read(ixx,*)xrecin(k),yrecin(k),zrecin(k)
c ---    Convert from METERS to KILOMETERS
         xrecin(k)=xrecin(k)*0.001
         yrecin(k)=yrecin(k)*0.001
      enddo

c --- Done with scratch file
      close(ixx)

c --- Configuration checks
      if(.NOT.LCONC) then
         k=47
         k1=k+1
         halt132(1:k)=
     &             'Halted in SETUP:  Must ONLY have Concentrations!'
         do i=k1,132
            halt132(i:i)=' '
         enddo
         write(ilst,*)halt132(1:k)
         return
      endif
      if(LDAYRANGE) then
         k=44
         k1=k+1
         halt132(1:k)=
     &             'Halted in SETUP:  DAYRANGE is not supported!'
         do i=k1,132
            halt132(i:i)=' '
         enddo
         write(ilst,*)halt132(1:k)
         return
      endif

c --- Final receptor check
      if(nrecin.LE.0) then
         k=37
         k1=k+1
         halt132(1:k)=
     &             'Halted in SETUP:  no receptors found!'
         do i=k1,132
            halt132(i:i)=' '
         enddo
         write(ilst,*)halt132(1:k)
         return
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine comp(halt132)
c----------------------------------------------------------------------
c
c --- AER2CAL    Version: 1.21     Level: 100301                  COMP
c                D. Strimaitis
c
c --- PURPOSE:  Process AERMOD POSTFILES to CALPUFF.DAT files
c
c  UPDATES:
c    v1.2 Level 091104  -->  v1.21 Level 100301  (DGS)
c         (1) Do AERMOD read with real*8 and then retry with real*4
c             if that produces an error
c
c    v1.03 Level 070925  -->  v1.2 Level 091104  (DGS)
c         (1) NEW AERMOD uses real*8 values so convert to real*4
c             after reading
c
c    v1.02 Level 070131  -->  v1.03 Level 070925  (DGS)
c         (1) Add HALT132 to control early terminations
c
c    v1.00 Level 061122  -->  v1.02 Level 070131  (DGS)
c         (1) Fix bug that allowed an extra 1-hour shift back for
c             all hours except the last hour of each day.  This bug
c             results in CALPOST output labels being off by 1 hour most
c             of the time.  (ENDIF in Condition hour 24 loop was placed
c             before the INCR call by mistake.)
c
c --- INPUT:  
c          HALT132 - character*132     - Status string
c
c          Common/CTRL/
c            nrecin,xrecin,yrecin,zrecin,atitle,aernam,nfil,
c            isyr,ismo,isday,isjul,ishr,ieyr,iemo,ieday,iejul,iehr
c            aerspec
c
c --- OUTPUT:  
c          HALT132 - character*132     - Status string
c
c
c --- COMP called by:  main
c --- COMP calls:      JULDAY, INCR, DELTT, WROUT1
c----------------------------------------------------------------------

c --- Include parameter statements
      include 'params.a2c'
      parameter (mxtype=1)

      include 'ctrl.a2c'
      include 'qa.a2c'

      character*132 concdat,blnk132
      character*15 speclev
      character*132 filnam132, halt132

      character*8 grpid
      character*16 csrcnam
      real aveval(mxdrec,mxtype)
      real*8 aveval8(mxdrec,mxtype)

      logical lreal8

c --- Source name must be TOTAL
      data csrcnam/'TOTAL           '/

c --- Conversion factor to CALPUFF g/m3
      data cfactor/1.0e-06/

c --- Seconds are zero
      data isec/0/

c --- Source configuration
      data ktype/0/, ksource/1/

c --- Set blnk132
      do i=1,132
         blnk132(i:i)=' '
      enddo

c --- Set species-level
      speclev=aerspec//'  1'

c --- Find range of x,y from receptors
      xmin=xrecin(1)
      xmax=xrecin(1)
      ymin=yrecin(1)
      ymax=yrecin(1)
      do i=2,nrecin
         if(xmin.GT.xrecin(i)) xmin=xrecin(i)
         if(ymin.GT.yrecin(i)) ymin=yrecin(i)
         if(xmax.LT.xrecin(i)) xmax=xrecin(i)
         if(ymax.LT.yrecin(i)) ymax=yrecin(i)
      enddo
      xc=0.5*(xmin+xmax)
      yc=0.5*(ymin+ymax)

c --- Set grid
      nx=2
      ny=2
      delxby2=0.5*(xmax-xmin)
      delyby2=0.5*(ymax-ymin)
      dgrid=AMAX1(delxby2,delyby2)
      x0=xc-dgrid
      y0=yc-dgrid

c --- Shift begin time to the start of the hour
      ibyr=isyr
      ibjul=isjul
      ibhr=ishr
      nhrinc=-1
      call INCR(ilst,ibyr,ibjul,ibhr,nhrinc,halt132)
      if(halt132(1:6) .NE. 'NORMAL') return

c --- Compute run length
      call DELTT(ibyr,ibjul,ibhr,ieyr,iejul,iehr,irlg)


c --- Loop over AERMOD files
c --------------------------
      numtyp=1
      do k=1,nfil
         write(*,*)'Processing: ',aernam(k)
         write(ilst,*)'Processing: ',aernam(k)
         open(in1,file=aernam(k),form='unformatted',status='old')

c ---    Create CALPUFF.DAT file name from AERMOD name by adding
c ---    '.cal' at the end
         klast=LEN_TRIM(aernam(k))
         concdat=blnk132
         concdat=aernam(k)(1:klast)//'.cal'
         open(io1,file=concdat,form='unformatted',status='unknown')

c ---    Write header records to file
         call WROUT1(ilst,io1,nx,ny,dgrid,x0,y0,
     &               ibyr,ibjul,ibhr,irlg,cfactor,
     &               nrecin,xrecin,yrecin,zrecin,
     &               ver,level,speclev,csrcnam,atitle)

c ---    Assume that AERMOD file uses real*8
         lreal8=.TRUE.

10       if(LREAL8) then
c ---       Read data record with double-precision values
            read(in1,end=999,iostat=ierr)
     &         kurdat,kave,grpid,((aveval8(irec,ityp),
     &         irec=1,nrecin),ityp=1,numtyp)
            if(ierr.NE.0) then
c ---          Try again with real*4
               write(ilst,*)'ERROR reading AERMOD file as R*8: ',
     &                      TRIM(aernam(k))
               write(ilst,*)'Retry as R*4'
               lreal8=.FALSE.
               REWIND(in1)
               goto 10
            endif
c ---       Scale concentrations to g/m3 and convert to R*4
            do j=1,nrecin 
               aveval(j,1)=cfactor*SNGL(aveval8(j,1))
            enddo
         else
c ---       Read data record with single-precision values
            read(in1,end=999,iostat=ierr)
     &         kurdat,kave,grpid,((aveval(irec,ityp),
     &         irec=1,nrecin),ityp=1,numtyp)
            if(ierr.NE.0) then
c ---          Something is wrong
               write(ilst,*)'ERROR reading AERMOD file as R*4: ',
     &                      TRIM(aernam(k))
               close(in1)
               close(io1)
               halt132='ERROR reading AERMOD file as R*4'
               return
            endif

c ---       Scale concentrations to g/m3
            do j=1,nrecin 
               aveval(j,1)=cfactor*aveval(j,1)
            enddo
         endif

c ---    DEBUG
c         write(*,*) 'kurdat,kave = ',kurdat,kave

c ---    Accept hourly data at this time
c         if(kave.NE.1) then
c            write(*,*)'COMP:  Non-hourly data encountered!'
c            write(*,*)'kurdat,kave,grpid = ',kurdat,kave,grpid
c            stop
c         endif

c ---    Extract date-time (kurdat is at end of hour)
         iyr2=kurdat/1000000
         imo2=kurdat/10000-iyr2*100
         idy2=kurdat/100-iyr2*10000-imo2*100
         ihr2=kurdat-iyr2*1000000-imo2*10000-idy2*100
c ---    YY to YYYY (current century)
         if(iyr2.LT.900 .AND. iyr2.GT.50) then
            iyr2=1900+iyr2
         elseif(iyr2.LT.50) then
            iyr2=2000+iyr2
         endif
         call JULDAY(ilst,iyr2,imo2,idy2,ijul2,halt132)
         if(halt132(1:6) .NE. 'NORMAL') return
c ---    Condition hour 24
         if(ihr2.EQ.24) then
            ihr2=23
            nhrinc=1
            call INCR(ilst,iyr2,ijul2,ihr2,nhrinc,halt132)
            if(halt132(1:6) .NE. 'NORMAL') return
         endif
c ---    Start of hour
         iyr1=iyr2
         ijul1=ijul2
         ihr1=ihr2
         nhrinc=-1
         call INCR(ilst,iyr1,ijul1,ihr1,nhrinc,halt132)
         if(halt132(1:6) .NE. 'NORMAL') return

c ---    Write data records without compression
         write(io1) iyr1,ijul1,ihr1,isec,iyr2,ijul2,ihr2,isec
         write(io1) ktype,ksource,csrcnam,xc,yc
         write(io1) speclev,(aveval(j,1),j=1,nrecin)

c ---    DEBUG
c         write(*,*) iyr1,ijul1,ihr1,isec,iyr2,ijul2,ihr2,isec
c         write(*,*) speclev,nrecin

         goto 10
999      close(in1)
         close(io1)
      
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine wrout1(io6,io8,nx,ny,dxkm,xorigkm,yorigkm,
     &                  iyrb,ijulb,ihrb,irlg,cfactor,
     &                  nrec,xrec,yrec,zrec,
     &                  ver,level,speclev,csrcnam,atitle)
c----------------------------------------------------------------------
c
c --- AER2CAL    Version: 1.21     Level: 080926                WROUT1
c                D. Strimaitis
c
c --- PURPOSE:  Write the header records to the concentration
c               file: CONC.DAT
c
c  UPDATES:
c    v1.01 Level 061124  -->  v1.04 Level 080926  (DGS)
c         - Change NWS-84 to WGS-84
c
c    v1.0  Level 061122  -->  v1.01 Level 061124  (DGS)
c         - Set AERMOD ver,level locally
c         - Restructure comment records
c
c --- WROUT1 called by:  COMP
c --- WROUT1 calls:      none
c----------------------------------------------------------------------

c --- Include parameter statements
      include 'params.a2c'
c
      real xrec(mxdrec),yrec(mxdrec),zrec(mxdrec)
c
      character*80 title(3),atitle(2)
      character*15 speclev
      character*16 csrcnam
c
      logical lsamp,lcomprs
c
      character*16 conset,dataver
      character*33 blank33
      character*64 datamod
      character*132 comment1,blank
c
      character*12 aver,alevel,mmodel
      character*12 ver,level
c
      character*4 utmhem
      character*8 datum,pmap
      character*12 daten
      character*16 clat0,clon0,clat1,clat2

      data iavg/1/,ione/1/
      data lsamp/.FALSE./, lcomprs/.FALSE./

      data blank33/'                                 '/

c --- Configure output documentation
      data mmodel/'AERMOD      '/
      data aver/'00000       '/
      data alevel/'00000       '/
      data conset/'CONC.DAT'/, dataver/'2.1'/
      data datamod/'No-Obs file structure with embedded control file'/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Complete the starting time specification
      nsecdt=3600
      isecb=0

c --- Set internal variables for output
c
      dykm=dxkm
      ibcomp=1
      iecomp=nx
      jbcomp=1
      jecomp=ny
      ibsamp=1
      iesamp=nx
      jbsamp=1
      jesamp=ny
      meshdn=1
c
      nssta=1
c
      msource=0
      npt1=1
      npt2=0
      nar1=0
      nar2=0
      nlines=0
      nln2=0
      nvl1=0
      nvl2=0
c
      nctrec=0
c
      nspout=1
      i2dmet=1
c
      xbtz=5
      iutmzn=19
      feast=0.0
      fnorth=0.0
      rnlat0=40.0
      relon0=-97.0
      xlat1=33.0
      xlat2=45.0
      pmap='UTM     '
      utmhem='N   '
      datum='WGS-84  '
      daten='02-21-2003  '
      clat0='40.0 N'
      clon0='97.0 W'
      clat1='33.0 N'
      clat2='45.0 N'
c

c --- Record #1 - File Declaration -- 24 words
      write(io8) conset,dataver,datamod
c
c --- Record #2 - Number of comment lines -- 1 word
      ncom=2
      write(io8) ncom
c
c --- Record #3 to NCOM+2 (Comment record section) -- 33 words each
      comment1=blank
      comment1(1:30)='Produced by AER2CAL Version:  '
c --- Construct the version-level comment string
      j=30
      do i=1,8
         if(ver(i:i).NE.' ') then
            comment1(j:j)=ver(i:i)
            j=j+1
         endif
      enddo
      j=j+1
      comment1(j:j+7)=' Level: '
      j=j+8
      do i=1,8
         if(level(i:i).NE.' ') then
            comment1(j:j)=level(i:i)
            j=j+1
         endif
      enddo
      write(io8) comment1

c --- Report COORDS version
      comment1=blank
      comment1(1:40)='NO Internal Coordinate Transformations  '
      write(io8) comment1
c
c --- HEADER RECORD #NCOM+3 -- General run parameters
      write(io8)mmodel,aver,alevel,iyrb,ijulb,ihrb,isecb,
     1 xbtz,irlg,iavg,nsecdt,nx,ny,dxkm,dykm,ione,xorigkm,yorigkm,nssta,
     2 ibcomp,iecomp,jbcomp,jecomp,ibsamp,jbsamp,iesamp,jesamp,
     3 meshdn,npt1,npt2,nar1,nar2,nlines,nln2,nvl1,nvl2,msource,
     4 nrec,nctrec,lsamp,nspout,lcomprs,i2dmet,
     5 iutmzn,feast,fnorth,rnlat0,relon0,xlat1,xlat2,
     6 pmap,utmhem,datum,daten,clat0,clon0,clat1,clat2
c
c --- HEADER RECORD #NCOM+4 -- Run title
      title(1)=atitle(1)
      title(2)=atitle(2)
      title(3)='Conversion Factor to g/m3:    '
      write(title(3)(31:50),'(e20.7)') cfactor
      write(io8)title
c
c --- HEADER RECORD #NCOM+5 -- List of species-groups output
c --- Set output concentration/layer label
      write(io8)speclev
c
c --- HEADER RECORD #NCOM+6 -- Discrete (non-gridded) receptor data
      if(nrec .NE. 0) write(io8) (xrec(i),i=1,nrec),
     *                           (yrec(i),i=1,nrec),
     *                           (zrec(i),i=1,nrec)
c
c --- HEADER RECORD #NCOM+7 -- Complex terrain receptor data
c     (nctrec = 0)
c
c --- HEADER RECORDs #NCOM+8 to 15 -- Source names
c     (not written to VISIBILITY files)
c     (Use dummy point source)
c
      if(npt1.GT.0) then
         itype=1
         write(io8) itype,csrcnam
      endif
c
c --- HEADER RECORDS
c ---     #NCOM+8 -- Nearest Surface Station for VISIBILITY ONLY
c ---     #NCOM+9 -- X coord (UTM) of stations for VISIBILITY ONLY
c ---    #NCOM+10 -- Y coord (UTM) of stations for VISIBILITY ONLY
c
      return
      end


c----------------------------------------------------------------------
      subroutine case320(io,mode,c320,nlim,halt132)
c----------------------------------------------------------------------
c
c --- AER2CAL   Version: 1.21     Level: 070925                CASE320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert all letters within a character string to 
c               either upper or lower case
c
c --- INPUTS:
c
c               IO - integer           - List file unit number
c             MODE - character*5       - Output directive for either 
c                                        upper or lower case
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c          HALT132 - character*132     - Status string
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c                                        uniform case
c          HALT132 - character*132     - Status string
c
c --- CASE320 called by: (utility)
c --- CASE320 calls:      IACHAR, CHAR
c----------------------------------------------------------------------

      character*320 c320
      character*132 halt132
      character*5 mode

      if(nlim.GT.320) then
         k=38
         k1=k+1
         k2=k+25
         k3=k2+1
         halt132(1:k)=
     &             'Halted in CASE320 - string is too long'
         halt132(k1:k2)=
     &             '   --   See the list file'
         do i=k3,132
            halt132(i:i)=' '
         enddo
         write(io,*)halt132(1:k)
         write(io,*)'Maximum length  : 320'
         write(io,*)'Requested length: ',nlim
         return
      endif

      if(mode.EQ.'UPPER' .OR. mode.EQ.'upper') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.97 .AND. ia.LE.122) then
               ia=ia-32
               c320(k:k)=CHAR(ia)
            endif
         enddo

      elseif(mode.EQ.'LOWER' .OR. mode.EQ.'lower') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.65 .AND. ia.LE.90) then
               ia=ia+32
               c320(k:k)=CHAR(ia)
            endif
         enddo
      else
         k=39
         k1=k+1
         k2=k+25
         k3=k2+1
         halt132(1:k)=
     &             'Halted in CASE320 - invalid output mode'
         halt132(k1:k2)=
     &             '   --   See the list file'
         do i=k3,132
            halt132(i:i)=' '
         enddo
         write(io,*) halt132(1:k)
         write(io,*)'Expected UPPER or LOWER'
         write(io,*)'Found: ',mode
         return
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine incr(io,iyr,ijul,ihr,nhrinc,halt132)
c----------------------------------------------------------------------
c
c --- AER2CAL   Version: 1.21     Level: 070925                    INCR
c               J. Scire
c
c --- PURPOSE:  Increment the time and date by "NHRINC" hours
c
c --- INPUTS:
c       IO     - integer - Unit number for list file output
c       IYR    - integer - Current year
c       IJUL   - integer - Current Julian day
c       IHR    - integer - Current hour (00-23)
c       NHRINC - integer - Time increment (hours)
c
c               NOTE: "NHRINC" must >= -24
c                      Hour is between 00-23
c
c      HALT132 - character*132  - Status string
c
c --- OUTPUT:
c       IYR    - integer - Updated year
c       IJUL   - integer - Updated Julian day
c       IHR    - integer - Updated hour (00-23)
c      HALT132 - character*132  - Status string
c
c --- INCR called by: host subroutines
c --- INCR calls:     none
c----------------------------------------------------------------------
      character*132 halt132
c
c --- Check nhrinc
      if(nhrinc.lt.-24) then
         k=40
         k1=k+1
         k2=k+25
         k3=k2+1
         halt132(1:k)=
     &             'Halted in INCR:  Invalid value of NHRINC'
         halt132(k1:k2)=
     &             '   --   See the list file'
         do i=k3,132
            halt132(i:i)=' '
         enddo
         write(io,*)halt132(1:k)//'-- NHRINC = ',nhrinc
         return
      endif

c --- Save increment remaining (needed if nhrinc > 8760)
      nleft=nhrinc
c
c --- Process change in hour
      if(nhrinc.gt.0)then
c
10       ninc=MIN0(nleft,8760)
         nleft=nleft-ninc
c
c ---    Increment time
         ihr=ihr+ninc
         if(ihr.le.23)return
c
c ---    Increment day
         ijul=ijul+ihr/24
         ihr=mod(ihr,24)
c
c ---    ILEAP = 0 (non-leap year) or 1 (leap year)
         if(mod(iyr,4).eq.0)then
            ileap=1
         else
            ileap=0
         endif
c
         if(ijul.gt.365+ileap) then
c ---       Update year
            iyr=iyr+1
            ijul=ijul-(365+ileap)
         endif
c
c ---    Repeat if more hours need to be added
         if(nleft.GT.0) goto 10
c
      elseif(nhrinc.lt.0)then
c ---    Decrement time
         ihr=ihr+nhrinc
         if(ihr.lt.0)then
            ihr=ihr+24
            ijul=ijul-1
            if(ijul.lt.1)then
               iyr=iyr-1
               if(mod(iyr,4).eq.0)then
                  ijul=366
               else
                  ijul=365
               endif
            endif
         endif
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine julday(io,iyr,imo,iday,ijuldy,halt132)
c----------------------------------------------------------------------
c
c --- AER2CAL   Version: 1.21     Level: 070925                  JULDAY
c ---            J. Scire
c
c --- PURPOSE:  Compute the Julian day number from the Gregorian
c               date (month, day)
c
c --- INPUTS:
c            IO - integer      - Unit number for list file output
c           IYR - integer      - Year
c           IMO - integer      - Month
c          IDAY - integer      - Day
c       HALT132 - character*132 - Status string
c
c --- OUTPUT:
c          IJUL - integer      - Julian day
c       HALT132 - character*132 - Status string
c
c --- JULDAY called by:  host subroutines
c --- JULDAY calls:      none
c----------------------------------------------------------------------
      character*132 halt132
c
      integer kday(12)
      data kday/0,31,59,90,120,151,181,212,243,273,304,334/
c
c --- Check for valid input data
      ierr=0
c --- Check for valid month
      if(imo.lt.1.or.imo.gt.12)ierr=1
c --- Check for valid day in 30-day months
      if(imo.eq.4.or.imo.eq.6.or.imo.eq.9.or.imo.eq.11)then
         if(iday.gt.30)ierr=1
      else if(imo.eq.2)then
         if(mod(iyr,4).eq.0)then
c ---       February in a leap year
            if(iday.gt.29)ierr=1
         else
c ---       February in a non-leap year
            if(iday.gt.28)ierr=1
         endif
      else
c ---    Check for valid day in 31-day months
         if(iday.gt.31)ierr=1
      endif
c
      if(ierr.eq.1)then
         k=31
         k1=k+1
         k2=k+25
         k3=k2+1
         halt132(1:k)=
     &             'Halted in JULDAY:  Invalid date'
         halt132(k1:k2)=
     &             '   --   See the list file'
         do i=k3,132
            halt132(i:i)=' '
         enddo
         write(io,*)halt132(1:k)//' - IYR = ',iyr,' IMO = ',
     1    imo,' IDAY = ',iday
         return
      endif
c
c --- Compute the Julian day
      ijuldy=kday(imo)+iday
      if(imo.le.2)return
      if(mod(iyr,4).EQ.0)ijuldy=ijuldy+1
c
      return
      end
c------------------------------------------------------------------------------
      subroutine deltt(j1yr,j1jul,j1hr,j2yr,j2jul,j2hr,jleng)
c------------------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.54     Level: 941215                  DELTT
c ---            J. Scire, SRC
c
c --- Compute the difference (in hours) between two dates & times
c ---    (time #2 - time #1)
c
c --- INPUTS:
c              J1YR - integer    - Year of date/time #1
c             J1JUL - integer    - Julian day of date/time #1
c              J1HR - integer    - Hour of date/time #1
c              J2YR - integer    - Year of date/time #2
c             J2JUL - integer    - Julian day of date/time #2
c              J2HR - integer    - Hour of date/time #2
c
c --- OUTPUT:
c             JLENG - integer    - Difference (#2 - #1) in hours
c
c --- DELTT called by:  host subroutines
c --- DELTT calls:      none
c------------------------------------------------------------------------------
c
      jmin=min0(j1yr,j2yr)
c
c --- find the number of hours between Jan. 1 of the "base" year and
c --- the first date/hour
      if(j1yr.eq.jmin)then
         j1=0
      else
         j1=0
         j1yrm1=j1yr-1
         do 10 i=jmin,j1yrm1
         if(mod(i,4).eq.0)then
            j1=j1+8784
         else
            j1=j1+8760
         endif
10       continue
      endif
      j1=j1+(j1jul-1)*24+j1hr
c
c --- find the number of hours between Jan. 1 of the "base" year and
c --- the second date/hour
      if(j2yr.eq.jmin)then
         j2=0
      else
         j2=0
         j2yrm1=j2yr-1
         do 20 i=jmin,j2yrm1
         if(mod(i,4).eq.0)then
            j2=j2+8784
         else
            j2=j2+8760
         endif
20       continue
      endif
      j2=j2+(j2jul-1)*24+j2hr
c
c --- compute the time difference (in hours)
      jleng=j2-j1
c
      return
      end

c----------------------------------------------------------------------
      subroutine comline(ctext)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.52     Level: 040330                COMLINE
c                J. Scire, SRC
c
c --- PURPOSE:  Call the compiler-specific system routine that will
c               pass back the command line argument after the text
c               that executed the program
c
c               This routine contains calls for several different
c               compilers, but only one should be active at any one
c               time.
c
c --- UPDATE
c --- V2.2 (960521) to V2.3 (040330)  DGS
c               - Replace strings c*70 with c*132
c
c --- INPUTS:
c
c          CTEXT - character*132 - Default command line argument #1
c
c --- OUTPUT:
c
c          CTEXT - character*132 - Command line argument #1
c                                  If command line argument is
c                                  missing, CTEXT is not changed
c
c --- COMLINE called by:  SETUP
c --- COMLINE calls:      GETCL - Lahey system routine
c                         NARGS, GETARG - Microsoft routines
c                         NARGS, GETARG - Compaq DF routines
c                         IARGC, GETARG - Sun routines
c                         IARGC, GETARG - HP routines
c----------------------------------------------------------------------
c
c ----------------------------------------
c --- COMPAQ DF compiler library directive
c *** USE DFLIB
c ----------------------------------------
c
      character*132 ctext,cdeflt
c
c --- Microsoft variables
c *** integer*2 iarg,istat
c
c --- HP declaration
c *** external getarg
c *** external iargc
c
c --- The following is for any system without a command line routine
c --- and is also used as a default
      cdeflt=ctext
c
c ------------------
c --- Lahey compiler
c ------------------
      call getcl(ctext)
c
c ----------------------
c --- COMPAQ DF compiler
c ----------------------
c ***  numargs=NARGS()
c ***  if(numargs.ge.1)then
c ***     call GETARG(1,ctext)
c ***  endif
c
c ----------------------
c --- Microsoft compiler
c ----------------------
c *** numargs=NARGS()
c *** if(numargs.ge.1)then
c ***    iarg=1
c ***    call GETARG(iarg,ctext,istat)
c *** endif
c
c ----------------
c --- Sun compiler
c ----------------
c *** numargs=IARGC()
c *** if(numargs.ge.1)then
c ***    call GETARG(1,ctext)
c *** endif
c
c -----------------------------------------------------
c --- HP compiler -- NOTE: needs +U77 switch on compile
c -----------------------------------------------------
c *** numargs=IARGC()
c *** if(numargs.ge.1)then
c ***    call GETARG(1,ctext)
c *** endif
c
c
c --- If no command line arguments, use default
      if(ctext(1:1).eq.' ')ctext=cdeflt

      return
      end
